/*
Titre: Exercice 2
Description: Un programme qui remplit un tableau d'entiers avec des nombres alatoires
 travers une fonction.

Auteur: Anis Boubaker
Date: 2018-02-15
*/

/***********COMMANDES PRPROCESSEUR*************/
#define _CRT_SECURE_NO_WARNINGS
#include <stdlib.h>
#include <stdio.h>

/***********CONSTANTES DE COMPILATION*************/
#define TAILLE_MAX_TABLEAU 40
#define MIN_ALEATOIRE 0
#define MAX_ALEATOIRE 100

/***************************************************************************************
*                              PROTOTYPES DE FONCTIONS                                  *
***************************************************************************************/
/*
* FONCTION: nb_aleatoire
Description: Cette fonction retourne une valeur alatoire dans l'intervalle entier
[min, max]. Chaque valeur possde la mme probabilit d'tre gnre.

PARAMTRES :
- min : La plus petite valeur pouvant tre gnre (type : int).
- max : La plus grande valeur pouvant tre gnre (type : int).

VALEUR DE RETOUR : Une valeur se trouvant dans l'intervalle
[min, max]  (type : int).

ANTECEDENTS :
- On suppose que la fonction srand a pralablement t appele.
*/
int nb_aleatoire(int min, int max);

/*
Fonction: remplir_aleatoire (Exercice 2)
Description: Remplit un tableau avec des nombres alatoires entre MIN_ALEATOIRE et 
MAX_ALEATOIRE.
Arguments:
- tableau (tableau d'entiers): Le tableau qui va contenir les nombres alatoires
- nb_elements: La taille du tableau.
Paramtres modifis:
- tableau: Le contenu du tableau sera remplaces par des valeurs alatoires
*/
void remplir_aleatoire(int tableau[], int nb_elements);

/*
Fonction: trouver_plus_petit_element (Exercice 3)
Description: Trouve la valeur du plus petit lment d'un tableau ainsi que sont indice
Arguments:
- tab (tableau d'entiers): LE tableau o on cherche le plus petit lment
- nb_elements (entier): Le nombre d'lments effectifs
- plus_petit_indice (pointeur d'entier): Adresse o sera stock l'indice du plus petit
lment
Paramtres modifis:
- plus_petit_indice: sa valeur sera remplace par l'indice du plus petit lment
*/
int trouver_plus_petit_element(int tab[], int nb_elements, int *plus_petit_indice);

/*
Fonction: compter_occurences (Exercice 4)
Description: Compte le nombre d'occurences d'un lment dans un tableau
Arguments:
- tableau (tableau d'entiers): Tableau o l'on compte le nombre d'occurences
- nb_elts (entier): Nombre d'lments effectifs du tableau
- elt_a_trouver (entier): lement recherch dans le tableau
Paramtres modifis:
- Aucun
*/
int compter_occurences(int tableau[], int nb_elts, int elt_a_trouver);

/*
Fonction: chercher_element (Exercice 5)
Description: Recherche un lment dans un tableau et renvoie son indice si trouv. 
Renvoie -1 si l'lment n'a pas t trouv
Arguments:
- tableau (tableau d'entiers): Tableau o l'on cherche l'lment
- nb_elts (entier): Nombre d'lments effectifs du tableau
- elt_a_trouver (entier): lement recherch dans le tableau
Paramtres modifis:
- Aucun
*/
int chercher_element(int tableau[], int nb_elts, int elt_a_trouver);

/*
Fonction: copier_tableau (Exercice 6)
Description: Copie un tableau dans un autre. Renvoie une valeur vraie si la copie s'est bien droule
et une valeur fausse sinon.
Arguments:
- tab_source (tableau d'entiers): Tableau source pour la copie
- nb_elts_source (entier): Nombre d'lments effectifs du tableau
- tab_dest (tableau d'entiers): Tableau source pour la copie
- max_elts_dest (entier): Taille maximale du tableau de destination
Paramtres modifis:
- tab_dest: Les lments du tableau seront crass par le contenu copi.
*/
int copier_tableau(int tab_source[], int nb_elts_source, int tab_dest[], int max_elts_dest);

/*
Fonction: renverser_tableau (Exercice 7)
Description: Inverse l'ordre des lments d'un tableau.
Arguments:
- tableau (tableau d'entiers): Tableau  inverser
- nb_elts (entier): Nombre d'lments effectifs du tableau
Paramtres modifis:
- tableau: Les lments du tableau seront inverss.
*/
void renverser_tableau(int tableau[], int nb_elets);

/***************************************************************************************
*                             FONCTIONS DE TESTS                                       *
***************************************************************************************/
int test_all(void);
int test_remplir_aleatoire(void);
int test_trouver_plus_petit_element(void);
int test_compter_occurences(void);
int test_chercher_element(void);
int test_copier_tableau(void);
int test_renverser_tableau(void);
/*
Fonction: main
*/

int main(void)
{
	//dclaration de variables
	int tab[TAILLE_MAX_TABLEAU] = { 0 };
	int i;
	int resultats_tests=1;

	//Initialisation du moteur de nombres alatoires avant d'appeler nb_aleatoire
	srand((unsigned int)time(NULL));
	rand();

	//Excution des tests
	test_all();

	system("pause");
	return EXIT_SUCCESS;
}


/********* IMPLMENTATIONS DES FONCTION **************/
int nb_aleatoire(int min, int max)
{
	return min + (int)(rand() / (RAND_MAX + 0.001) * (max - min + 1));
}


void remplir_aleatoire(int tableau[], int nb_elements) {
	int i; 
	for (i = 0; i < nb_elements; i++)
	{
		tableau[i] = nb_aleatoire(MIN_ALEATOIRE, MAX_ALEATOIRE);
	}
}

int test_remplir_aleatoire(void)
{
	int mon_tab[TAILLE_MAX_TABLEAU] = { 0 };
	int variable_de_copie;
	int i; //compteur de boucles

	printf("*** TEST: remplir_aleatoire\n");

	//On remplit le tableau de -1 et on vrifie que toutes les valeurs ont chang
	for (i = 0; i < TAILLE_MAX_TABLEAU; i++) {
		mon_tab[i] = -1;
	}

	remplir_aleatoire(mon_tab, TAILLE_MAX_TABLEAU);

	for (i = 0; i < TAILLE_MAX_TABLEAU; i++) {
		if (mon_tab[i] == -1) {
			printf("*** ERREUR: le tableau n'a pas t intgralement remplit de nouvelles valeurs.");
			return 0;
		}
	}

	//On teste avec une taille nulle
	variable_de_copie = mon_tab[0];
	remplir_aleatoire(mon_tab, 0);
	if (mon_tab[0] != variable_de_copie)
	{
		printf("*** ERREUR: Remplir tableau avec une taille nulle.");
		return 0;
	}
	return 1;
}

int trouver_plus_petit_element(int tab[], int nb_elements, int *plus_petit_indice)
{
	int i; //compteur de boucle
	int plus_petit = INT_MAX;

	//-1: valeur du + petit indice si le tableau est vide.
	*plus_petit_indice = -1;

	for (i = 0; i < nb_elements; i++)
	{
		if (tab[i] < plus_petit)
		{
			plus_petit = tab[i];
			*plus_petit_indice = i;
		}
	}

	return plus_petit;
}

int test_trouver_plus_petit_element()
{
	int tableau[5] = { 56,6,23,2,10 };
	int min,
		min_indice;

	printf("*** TEST: trouver_plus_petit_element\n");

	min = trouver_plus_petit_element(tableau, 5, &min_indice);

	if (min != 2 || min_indice != 3)
	{
		printf("*** ERREUR: Valeur min ou indice min incorrect.");
		return 0;
	}

	min = trouver_plus_petit_element(tableau, 1, &min_indice);
	if (min != 56 || min_indice != 0)
	{
		printf("*** ERREUR: Incoherence d'execution avec un tableau ayant une seule valeur.\n");
		return 0;
	}

	min = trouver_plus_petit_element(tableau, 0, &min_indice);
	if (min != INT_MAX || min_indice != -1)
	{
		printf("*** ERREUR: Incoherence d'execution avec un tableau vide.\n");
		return 0;
	}

	return 1;
}

int compter_occurences(int tableau[], int nb_elts, int elt_a_trouver)
{
	int i;  //Compteur de boucle
	int nb_occurences = 0; //Stocke le nombre d'occurences de elt_a_trouver

	for (i = 0; i < nb_elts; i++)
	{
		if (tableau[i] == elt_a_trouver)
		{
			nb_occurences++;
		}
	}
	return nb_occurences;
}

int test_compter_occurences(void)
{
	int tab[5] = { 10, 34, 2, 78, 10 };
	int occurences;

	printf("*** TEST: compter_occurences\n");

	occurences = compter_occurences(tab, 5, 10);
	if (occurences != 2)
	{
		printf("*** ERREUR: Erreur de calcul du nombre d'occurence. Attendu: 2, Obtenu: %d.\n", occurences);
		return 0;
	}

	occurences = compter_occurences(tab, 0, 10);
	if (occurences != 0)
	{
		printf("*** ERREUR: Erreur de calcul du nombre d'occurence avec tableau vide.\n");
		return 0;
	}

	occurences = compter_occurences(tab, 5, -1);
	if (occurences != 0)
	{
		printf("*** ERREUR: Erreur de calcul du nombre d'occurence avec valeur inexistante.\n");
		return 0;
	}

	return 1;
}

int chercher_element(int tableau[], int nb_elts, int elt_a_trouver)
{
	int i; 

	for (i = 0; i < nb_elts; i++)
	{
		if (tableau[i] == elt_a_trouver)
		{
			return i;
		}
	}
	
	return -1;
}

int test_chercher_element(void)
{
	int tab[5] = { 10, 34, 2, 34, 10 };
	int indice;

	printf("*** TEST: chercher_element\n");

	indice = chercher_element(tab, 5, 2);
	if (indice != 2)
	{
		printf("*** ERREUR: Indice obtenu erronn. Attendu: 2, Obtenu: %d.\n", indice);
		return 0;
	}

	indice = chercher_element(tab, 0, 10);
	if (indice != -1)
	{
		printf("*** ERREUR: Erreur de recherche dans un tableau vide.\n");
		return 0;
	}

	indice = chercher_element(tab, 5, -1);
	if (indice != -1)
	{
		printf("*** ERREUR: Erreur de recherche d'une valeur inexistante.\n");
		return 0;
	}

	indice = chercher_element(tab, 5, 34);
	if (indice != 1)
	{
		printf("*** ERREUR: Indice erronn lors de recherche d'une valeur qui existe plus d'une fois dans le tableau.\n");
		return 0;
	}

	return 1;
}


int copier_tableau(int tab_source[], int nb_elts_source, int tab_dest[], int max_elts_dest)
{
	int i; //compteur de boucle

	if (nb_elts_source > max_elts_dest)
	{ //Copie impossible car le tableau de destination est trop petit.
		return 0;
	}

	for (i = 0; i < max_elts_dest; i++)
	{
		if (i < nb_elts_source)
		{
			tab_dest[i] = tab_source[i];
		} 
		else
		{
			tab_dest[i] = 0;
		}
	}

	return 1;
}

int test_copier_tableau(void)
{
	int tab1[5] = { 10, 34, 2, 34, 10 };
	int tab2[8] = { 10, 34, 2, 34, 10, 23, 10, 6 };
	int indice;

	printf("*** TEST: copier_tableau\n");

	return 1;
}

void renverser_tableau(int tableau[], int nb_elets)
{
	int i; //Compteur de boucle
	int copie; //Variable de copie pour les permutations
	
	for (i = 0; i < nb_elets / 2; i++)
	{
		copie = tableau[i];
		tableau[i] = tableau[nb_elets - 1 - i];
		tableau[nb_elets - 1 - i] = copie;
	}

}

int test_renverser_tableau(void)
{
	int tab1[5] = { 1, 2, 3, 4, 5 };
	printf("*** TEST: renverser_tableau\n");

	renverser_tableau(tab1, 5);
	if (tab1[0] != 5 || tab1[1] != 4 || tab1[2] != 3 || tab1[3] != 2 || tab1[4] != 1) {
		printf("*** ERREUR: Les valeurs du tableau n'ont pas t renverses correctement.\n");
		return 0;
	}

	renverser_tableau(tab1, 0);
	if (tab1[0] != 5 || tab1[1] != 4 || tab1[2] != 3 || tab1[3] != 2 || tab1[4] != 1) {
		printf("*** ERREUR: Inversion incorecte d'un tableau vide..\n");
		return 0;
	}

	return 1;
}

int test_all(void)
{
	int resultats_tests = 1;

	//Excution des tests
	resultats_tests = resultats_tests && test_remplir_aleatoire();
	resultats_tests = resultats_tests && test_trouver_plus_petit_element();
	resultats_tests = resultats_tests && test_compter_occurences();
	resultats_tests = resultats_tests && test_chercher_element();
	resultats_tests = resultats_tests && test_copier_tableau();
	resultats_tests = resultats_tests && test_renverser_tableau();

	if (resultats_tests)
	{
		printf("***\n*** Tous les tests se sont executes avec succes.\n***\n");
	}
	else {
		printf("***\n*** Des erreurs se sont produites lors de l'execution des tests\n***\n");
	}

	return resultats_tests;
}